<?php

namespace Controller;

use Panier;
use Texte;
use Produit;
use Attribut;
use DediException;
use Pays;
use Boutique;
use Promotion;
use Client;
use Breadcrumb;
use Reduction;
use Livraison;
use BonAchat;
use Adresse;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class CartController
 * @package Controller
 */
class CartController extends Controller
{

    /**
     * @Route(
     *      path = "/shopping_cart.html",
     *      name = "panier"
     * )
     *
     * @return mixed
     *
     * @throws DediException
     */
    public function show()
    {

        $session = $this->request->getSession();

        $panier = Panier::getInstance();
        $pays = new Pays;
        $promotion = new Promotion;
        $texte = new Texte();
        $bd = Breadcrumb::getInstance();

        if ($session->has('client_id')) {
            $customer = $panier->getCustomer();
            if (empty($customer)) {
                $panier->setCustomer(new Client($session->get('client_id')));
            }
        }

        // si code promo, on check si toujours valide
        $promotion->checkReduction();

        list($totalHT, $totalTTC) = $panier->getTotalProducts(false);

        // affectation du pays de livraison
        // soit le pays de langue par defaut, soit le dernier pays selectionne
        $paysId = $this->request->get('pays_id');
        if (empty($paysId)) {
            $client = $this->getCurrentClient();
            if ($client) {
                $adresse = new Adresse();
                $arrAdresse = $adresse->get(array("client_id" => $this->getCurrentClient()->client_id, 'order' => 'adresse_default_livraison DESC, adresse_id DESC'));
                if (!empty($arrAdresse)) {
                    $paysId = $arrAdresse[0]['pays_id'];
                }
            }
            if (empty($paysId)) {
                $paysId = $pays->get(array('pays_iso_code_2' => Boutique::getInstance()->getLangue()->ini), 'pays_id');
            }
        }

        // on regarde si le panier est TTC ou HT
        $this->template->assign('isTva', $pays->isTva($paysId));

        if ($panier->have('produit_type_port', 1) == true) {

            if ($this->request->get('livraison_id') != false) {
                $panier->calculateShipping($this->request->get('livraison_id'));
            }

            // TODO : voir pourquoi on a pas de mode de livraison
            $ship = $panier->getShipping();
            if (empty($ship)) {
                $panier->calculateShipping();
                $ship = $panier->getShipping();
            }
            $this->template->assign('port', $ship->toArray());
        }

        $livraison = new Livraison();
        $arrLivraison = $livraison->get(array(
            'pays_id' => $paysId,
            'poids' => $panier->getWeight(),
            'prix' => $totalHT,
            'livraison_type_id' => array(Livraison::TYPE_DOM, Livraison::TYPE_REL)
        ));
        if ($arrLivraison) {
            $livraisons = array();
            foreach ($arrLivraison as $livraison) {
                $livraisons[$livraison['livraison_id']] = $livraison['livraison_nom_nom'];
            }
            $this->template->assign('livraisons', $livraisons);
            $this->template->assign('livraisonSelected', $this->request->get('livraison_id'));
        }

        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();

        $reduction = $panier->getReduction();
        if (!empty($reduction)) {
            $this->template->assign('montantReduc', $reduction->price);
            $this->template->assign('codePromo', $reduction->code);
        }

        $this->template->assign('aPayerHT', $aPayerHT);
        $this->template->assign('aPayerTTC', $aPayerTTC);
        $this->template->assign('aPayerTVA', ($aPayerTTC - $aPayerHT));

        $this->template->assign('nextPage', Boutique::getInstance()->getNextPageProcess());

        // chargement des infos panier
        $this->template->assign('caddie', $panier->getProducts());

        // Recuperation des pays
        $this->template->assign('pays', $pays->get(array('format' => 'assoc')));
        $this->template->assign('paysSelected', $paysId);

        $this->template->assign('sTotalTTC', $totalTTC);
        $this->template->assign('sTotalHT', $totalHT);

        $bd->add($texte->getOneOrCreate('mon_panier', 'Mon Panier'));

        return $this->render('panier.tpl');
    }

    /**
     * @Route("/cart/rollover", name="cart_rollover")
     */
    public function rollover()
    {
        $panier = Panier::getInstance();
        $shipping = $panier->getShipping();

        $this->template->assign('countProducts', $panier->getCountProducts());
        $this->template->assign('port', (!empty($shipping)) ? $shipping->toArray() : array());
        $this->template->assign('caddieHeader', $panier->getProducts());

        $aPayer = $panier->getAPayer();
        $this->template->assign('aPayer', $aPayer);
        $this->template->assign('montantTva', $aPayer[1] - $aPayer[0]);
        list($totalHT, $totalTTC) = $panier->getTotalProducts();

        $this->template->assign('totalHT', $totalHT);
        $this->template->assign('totalTTC', $totalTTC);

        return $this->response(
            $this->template->fetch('cart/rollover.tpl')
        );
    }

    /**
     * @Route(
     *      path         = "/cart/add",
     *      name         = "add_cart",
     *      methods      = { "POST" }
     * )
     *
     * @return mixed
     *
     * @throws DediException
     * @throws \Exception
     * @throws \SmartyException
     */
    public function add()
    {
        $panier = Panier::getInstance();
        $attribut = null;

        $error = array();
        $texte = new Texte();

        $json = array();

        // cas ajout au panier
        if ($this->request->get('produit_id') != false) {
            $produit = new Produit($this->request->get('produit_id'));
        } else {
            $error[] = $texte->getOneOrCreate('noProductId', '');
        }

        $povId = $this->request->get('produit_option_valeur_id');
        $povIdBis = $this->request->get('produit_option_valeur_id_bis');

        if ($this->request->get('produit_attribut_id') != false) {
            $attId = $this->request->get('produit_attribut_id');

            if (empty($attId) || $attId == -1) {
                if (empty($povId) && empty($povIdBis)) {
                    $error[] = $texte->getOneOrCreate('noAttrSelect', 'Vous devez selectionner une taille et une couleur');
                } elseif (empty($povId)) {
                    $error[] = $texte->getOneOrCreate('noTailleSelect', 'Vous devez selectionner une taille');
                } elseif (empty($povIdBis)) {
                    $error[] = $texte->getOneOrCreate('noCouleurSelect', 'Vous devez selectionner une couleur');
                }
            } else {
                $attribut = new Attribut($attId);
            }
        }

        $quantite = ($this->request->get('quantite') != false) ?  $this->request->get('quantite') : 1;

        if (count($error) == 0) {

            try {
                $identifier = $panier->addProduct($produit, $attribut, $quantite);
            } catch (DediException $e) {
                $error[] = $e->getMessage();
            }
        }

        // si erreur : on recup les infos du produit, pour que le client complete la selection de ses options dans la popup
        if (count($error) == 0 && !empty($identifier)) {
            $product = $panier->getProductsByIdentifier($identifier);
            $json['status'] = true;
            $json['countProduct'] = $panier->getCountProducts();
            $json['message'] =  $texte->replaceTag($texte->getOneOrCreate('success_ajout_panier', 'Votre produit %PRODUIT_NOM% a été ajouté au panier'), array(
                'produit_nom' => $product['marque_nom'] . ' - ' . $product['produit_nom_nom']
            ));
        } else {
            $json['status'] = false;
            $json['error'] = $error;
        }

        return $this->response($json);
    }

    /**
     * @Route("/cart/update", name="update_cart", methods = { "POST" })
     *
     * @return mixed
     */
    public function update()
    {
        $json = array();

        if ($this->request->get('identifier') == null) {
            $json['status'] = false;
        } else {
            $panier = Panier::getInstance();
            try {

                $panier->updateProduct(
                    $this->request->get('identifier'),
                    $this->request->get('quantite')
                );

                $json['status'] = true;
            } catch (DediException $e) {
                $json['status'] = false;
                $json['error'] = $e->getMessage();
            }
        }

        return $this->response($json);
    }

    /**
     * @Route("/cart/delete", name="delete_cart", methods = { "POST" })
     *
     * @return mixed
     */
    public function delete()
    {
        $json = array();

        if ($this->request->get('identifier') == null) {
            $json['status'] = false;
        } else {
            $panier = Panier::getInstance();
            try {
                $panier->deleteProduct($this->request->get('identifier'));
                $json['status'] = true;
            } catch (DediException $e) {
                $json['status'] = false;
                $json['error'] = $e->getMessage();
            }
        }

        return $this->response($json);
    }

    /**
     * @Route(
     *      path         = "/cart/reduction/add",
     *      name         = "add_reduction_cart",
     *      methods      = { "POST" }
     * )
     *
     * @return mixed
     */
    public function addReduction()
    {
        $json = array();

        $error = array();
        $texte = new Texte();

        if ($this->request->get('promotion_code') == null) {
            $json['status'] = false;
            $error[] = $texte->getOneOrCreate('codePromoEmpty', 'Vous devez renseigner un code promo.');
            $json['message'] = implode('<br />', $error);
        } else {
            $panier = Panier::getInstance();
            $promotion = new Promotion();

            if ($this->request->get('promotion_code') == false) {
                $panier->initReduction();
                $json['status'] = true;
            } else {
                try {
                    $reduction = $promotion->apply($this->request->get('promotion_code'));
                } catch (DediException $e) {
                    $error[] = $e->getMessage();
                    $json['status'] = false;
                }

                if (isset($reduction) && $reduction == false) {
                    $json['status'] = false;
                    $error[] = $texte->getOneOrCreate('codePromoWrong' , 'Vous ne pouvez pas utiliser ce code promo.');
                }

                if (count($error) > 0) {
                    $json['message'] = implode('<br />', $error);
                } else {
                    if (isset($reduction) && $reduction instanceof Reduction) {
                        $panier->initReduction();
                        $panier->addReduction($reduction);
                        $json['status'] = true;
                    }
                }
            }
        }

        return $this->response($json);
    }

    /**
     * @Route(
     *      path         = "/cart/reduction/delete",
     *      name         = "delete_reduction_cart",
     *      methods      = { "POST" }
     * )
     *
     * @return mixed
     */
    public function deleteReduction()
    {
        $promotion_code = $this->request->get('promotion_code');
        $redirect = $this->request->get('redirect');

        if ($promotion_code != null) {

            $panier = Panier::getInstance();
            $reduction = new Reduction();
            $reduction->code = $promotion_code;

            $panier->deleteReduction($reduction);
            $panier->initReduction();
        }

        if (!empty($redirect)){
            return $this->redirect($redirect);
        }

        return $this->redirect($this->generateUrl('panier'));
    }

    /**
     * @Route("/cart/shipping/select", name="select_shipping_cart", methods = { "POST" })
     *
     * @return mixed
     */
    public function selectShipping()
    {

        $json = array('status' => false, 'relais' => '');
        $panier = Panier::getInstance();

        if ($this->request->get('livraison_zone_id') != false) {

            $isTva = $panier->isTva();

            $poidsPanier = $panier->getWeight();
            list($totalHT, $totalTTC) = $panier->getTotalProducts();

            $livraison = new Livraison();
            $addrShipping = $panier->getAddressShipping();
            $paramLivr = array(
                'pays_id' => $addrShipping->pays_id,
                'poids' => $poidsPanier,
                'prix' => $totalHT,
                'livraison_zone_id' => $this->request->get('livraison_zone_id'),
            );
            if ($isTva) {
                $paramLivr['prix_ttc'] = $totalTTC;
            } else {
                $paramLivr['prix_ht'] = $totalHT;
            }
            $arrPort = $livraison->get($paramLivr);

            if (!empty($arrPort)) {
                $livraison->hydrate($arrPort[0]);
                $panier->setShipping($livraison);
                $json['status'] = true;

                $devise = Boutique::getInstance()->getDevise();

                $json['data_bind'] = array();
                $json['data_bind']['port'] = ($isTva) ? $livraison->port_ttc : $livraison->port_ht;
                $json['data_bind']['port'] = $devise->format($json['data_bind']['port']);

                list($aPayerHT, $aPayerTTC) = $panier->getAPayer();
                $json['data_bind']['apayer'] = ($isTva) ? $aPayerTTC : $aPayerHT;
                $json['data_bind']['apayer'] = $devise->format($json['data_bind']['apayer']);

                list($totalHT, $totalTTC) = $panier->getTotalProductsAndShip();
                $json['data_bind']['total'] = ($isTva) ? $totalTTC : $totalHT;
                $json['data_bind']['total'] = $devise->format($json['data_bind']['total']);

                // chargement point relais si necessaire
                if (Livraison::isRelay($livraison)) {
                    $json['relais'] = $livraison->getHTMLRelay(
                        $livraison->livraison_class,
                        array(
                            'ville' => $addrShipping->adresse_ville,
                            'code_postal' => $addrShipping->adresse_cp,
                            'pays' => $addrShipping->pays_iso_code_2
                        )
                    );
                }
            }
        }

        return $this->response($json);
    }

    /**
     * @Route("/cart/voucher/add", name="add_voucher_cart", methods = { "POST" })
     *
     * @return mixed
     */
    public function addVoucher()
    {
        $json = array('status' => false);

        if ($this->request->get('bon_achat_id') == null) {
            $json['status'] = false;
        } else {
            $panier = Panier::getInstance();

            $ba = new BonAchat($this->request->get('bon_achat_id'));
            $error = $ba->isValid();

            if (count($error) == 0) {
                $panier->addBonAchat($ba);
                $json['status'] = true;
            } else {
                $json['error'] = implode('<br />', $error);
            }
        }

        return $this->response($json);
    }

    /**
     * @Route("/cart/voucher/delete", name="delete_voucher_cart", methods = { "POST" })
     *
     * @return mixed
     */
    public function deleteVoucher()
    {
        $json = array('status' => false);

        if ($this->request->get('bon_achat_id') == null) {
            $json['status'] = false;
        } else {
            $panier = Panier::getInstance();

            try {
                $panier->deleteBonAchat(new BonAchat($this->request->get('bon_achat_id')));
                $json['status'] = true;
            } catch (DediException $e) {
                $json['error'] = $e->getMessage();
            }
        }

        return $this->response($json);
    }

    /**
     * @Route("/cart/address/change", name="change_address_cart", methods = { "POST" })
     *
     * @return mixed
     */
    public function changeAddress()
    {
        $json = array('status' => false);

        if ($this->request->get('adresse_id') == null || $this->request->get('type') == null) {
            $json['status'] = false;
        } else {
            $panier = Panier::getInstance();

            try {
                $addr = new Adresse($this->request->get('adresse_id'));
                if ($this->request->get('type') == 'shipping') {
                    $panier->setAddressShipping($addr);
                    if ($panier->getSameAddress() == true) {
                        $panier->setAddressBilling(clone $addr);
                    }
                } else {
                    $panier->setAddressBilling($addr);
                }
                $json['status'] = true;
            } catch (DediException $e) {
                $json['error'] = $e->getMessage();
            }
        }

        return $this->response($json);
    }

    /**
     * @Route("/cart/same-address", name="same_address_cart", methods = { "POST" })
     *
     * @return mixed
     */
    public function setSameAddress()
    {
        $json = array('status' => false);

        if ($this->request->get('same_address') == null) {
            $json['status'] = false;
        } else {
            $panier = Panier::getInstance();

            try {
                $panier->setSameAddress($this->request->get('same_address'));
                $json['status'] = true;
            } catch (DediException $e) {
                $json['error'] = $e->getMessage();
            }
        }

        return $this->response($json);
    }
}
