<?php

namespace Controller;

use Form\NewsletterForm;
use Symfony\Component\Routing\Annotation\Route;
use Newsletter;
use Boutique;
use Texte;

/**
 * class NewsletterController
 *
 * @package Controller
 *
 * @Route(path = "/newsletter")
 */
class NewsletterController extends Controller
{

    /**
     * Function call when someone want to subscribe to the newsletter
     *
     * @Route("/subscribe", name="newsletter_subscribe")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     *
     * @throws \DediException
     */
    public function subscribe()
    {
        $form = new NewsletterForm();
        $newsletter = new Newsletter();
        $texte = new Texte();
        $json = array();

        $flashBag = $this->getFlashBag();

        if ($error = $form->handle($this->request)) {
            $flashBag->add("error_newsletter", $error);
        }

        if ($flashBag->has("error_newsletter") == false) {
            $boutique = Boutique::getInstance();

            $data = [
                'newsletter_email' => $this->request->get('email'),
                'boutique_id' => $boutique->id,
                'langue_id' => $boutique->getLangue()->id,
            ];

            $id = $newsletter->save($data);

            $json['status'] = true;
            $flashBag->add("success_newsletter",$texte->getOneOrCreate("newsletter_success","Votre demande d'inscription à la newsletter a bien été prise en compte"));
        }

        $error = $flashBag->get('error_newsletter', []);
        $success = $flashBag->get('success_newsletter', []);
        $json['html'] = $this->template->fetch('form/flashContainer.tpl', compact('error', 'success'));

        return $this->response($json);
    }
}