<?php

namespace Controller;

use Application;
use Breadcrumb;
use Client;
use Panier;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Adresse;
use Livraison;
use Paiement;
use Symfony\Component\Routing\Exception\ResourceNotFoundException;
use Texte;
use Tpe;
use Commande;
use DediException;
use Avoir;
use BonAchat;
use Pays;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class OrderController
 * @package Controller
 */
class OrderController extends Controller {

    /**
     * @Route(
     *      path         = "/checkout_order.html",
     *      name         = "order",
     *      methods      = { "GET" },
     *      defaults     = {"role" = "customer"}
     * )
     *
     * @return mixed
     */
    public function show()
    {

        // TODO : check device, si mobile => redir vers process en 4 etapes !

        $texte = new Texte();
        $bd = Breadcrumb::getInstance();
        $bd->add($texte->getOneOrCreate('mon_panier', 'Mon panier'));
        $bd->add($texte->getOneOrCreate('recapitulatif', 'Récapitulatif'));

        $panier = Panier::getInstance();
        $session = $this->request->getSession();
        $tpl_vars = array();
        $bonAchat = new BonAchat();
        $promotion = new \Promotion;

        if ($panier->getCountProducts(false) == 0) {
            return new RedirectResponse($this->generateUrl('panier'));
        }

        $adresse = new Adresse();
        $livraison = new Livraison();

        $promotion->checkReduction();

        list($sTotalHT, $sTotalTTC) = $panier->getTotalProducts();
        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();

        $panier->initRelay();


        $client = new Client($session->get('client_id'));
        $addrDefault = $adresse->getDefault($client->client_id);

        // init addresse
        if (!empty($addrDefault)) {
            $addrShip = $panier->getAddressShipping();
            if (empty($addrShip)) {
                $panier->setAddressShipping($addrDefault);
            }

            $addrBill = $panier->getAddressBilling();
            if (empty($addrBill)) {
                $panier->setAddressBilling($addrDefault);
            }
        }

        $addresses = array(
            'shipping' => $panier->getAddressShipping(),
            'billing' => $panier->getAddressBilling(),
        );

        $weight = $panier->getWeight();

        // recup produits panier
        $caddie = $panier->getProducts();

        // recup livraison
        $shipping = $panier->getShipping();

        /*if (Livraison::isRelay($shipping)) {

            $relais = $livraison->getHTMLRelay(
                $shipping->livraison_class,
                array('ville' => $addresses['shipping']->adresse_ville, 'code_postal' => $addresses['shipping']->adresse_cp)
            );
            $this->template->assign('relais', $relais);
        }*/

        $paramLivr = array();
        $paramLivr['pays_id'] = $addresses['shipping']->pays_id;
        $paramLivr['poids'] = $weight;
        $paramLivr['prix'] = $sTotalHT;

        if ($panier->isTva()) {
            $paramLivr['prix_ttc'] = $sTotalTTC;
        } else {
            $paramLivr['prix_ht'] = $sTotalHT;
        }
        $livraisons = $livraison->get($paramLivr);
        if(!empty($livraisons)){
            foreach($livraisons as &$livr){
                $livr['delai_min_date'] = $panier->getDelai($livr['livraison_delai_min']);
                $livr['delai_max_date'] = $panier->getDelai($livr['livraison_delai_max']);
            }
        }

        // recup mode de paiement
        $paiement = new Paiement();
        $paiements = $paiement->get(array('prix' => $aPayerHT));


        $reduction = $panier->getReduction();
        if (!empty($reduction)) {
            $this->template->assign('codePromo', $reduction->code);
            $this->template->assign('montantReduc', $reduction->price);
        }

        // on recup les ba valide et qui peuvent etre utilisé
        $this->template->assign('bonAchats', $bonAchat->get(array(
            'client_id' => $client->client_id,
            'bon_achat_statut_id' => BonAchat::BA_VALIDE,
            'interval' => array(date('Y-m-d'))
        )));

        // on recup les BA deja selectionner
        $baSelected = $panier->getBonAchat();
        $listBaSelected = array();
        $totalBa = 0;
        if (!empty($baSelected)) {
            foreach ($baSelected as $baS) {
                $listBaSelected[] = $baS->bon_achat_id;
                $totalBa+= $baS->bon_achat_prix;
            }
        }


        $this->template->assign('listBaSelected', $baSelected);
        $this->template->assign('bonAchatsSelected', $listBaSelected);
        $this->template->assign('totalBa', $totalBa);

        // Assign variables
        $addresses['billing'] = $addresses['billing']->toArray();
        $addresses['shipping'] = $addresses['shipping']->toArray();
        $this->template->assign('addrBilling', $addresses['billing']);
        $this->template->assign('addrShipping', $addresses['shipping']);
        $this->template->assign('addr', $addresses);
        $this->template->assign('addresses', $adresse->get(array('client_id' => $client->client_id)));
        $this->template->assign('port', $shipping->toArray());

        $this->template->assign('livraisons', $livraisons);
        $this->template->assign('paiements', $paiements);

        list($totalHT, $totalTTC) = $panier->getTotalProducts(false);
        $this->template->assign('sTotalTTC', $totalTTC);
        $this->template->assign('sTotalHT', $totalHT);

        $this->template->assign('aPayerHT', $aPayerHT);
        $this->template->assign('aPayerTTC', $aPayerTTC);
        $this->template->assign('montantTva', $aPayerTTC - $aPayerHT);

        $this->template->assign('caddie', $caddie);
        $this->template->assign('sameAddress', $panier->getSameAddress());

        $this->template->assign('civilite', $client->getCivilite());
        $this->template->assign('pays', (new Pays())->get());

        $flashBag = $this->getFlashBag();
        if ($flashBag->has('error')) {
            $tpl_vars['error'] = $flashBag->get('error');
        }

        return $this->render('order.tpl', $tpl_vars);
    }

    /**
     * @Route(
     *      path         = "/checkout_order.html",
     *      name         = "order_post",
     *      methods      = { "POST" },
     *      defaults     = { "role" = "customer" }
     * )
     *
     * @return RedirectResponse
     *
     * @throws DediException
     */

    public function valid()
    {
        $texte = new Texte();
        $livraison = new Livraison();
        $panier = Panier::getInstance();
        $session = $this->request->getSession();
        $flashBag = $this->getFlashBag();

        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();

        if ($this->request->get('livraison_zone_id') == false) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errSelectLivraison', 'Vous devez selectionner un mode de livraison')
            );
        } else {

            $livraisonSelect = new Livraison();
            $dataLivraisonSelect = $livraison->get(['livraison_zone_id' => $this->request->get('livraison_zone_id')]);
            $livraisonSelect->hydrate($dataLivraisonSelect[0]);

            if (Livraison::isRelay($livraisonSelect)) {
                if ($this->request->get('relay_id') == false) {
                    $flashBag->add('error', 'noRelaySelected');
                } else {
                    $oRelais = $livraison->getObjectRelay($livraisonSelect->livraison_class);
                    $relay = $oRelais->getRelay($this->request->get('relay_id'));
                    $panier->setRelay($relay);
                }
            }
        }

        if ($this->request->get('paiement_id') == false && $aPayerHT > 0) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errSelectPaiement', 'Vous devez selectionner un mode de paiement')
            );
        }

        if ($this->request->get('cgv') == false) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errValidCGV', 'Vous devez valider les CGV pour passer au paiement')
            );
        }

        if ($flashBag->has('error') == false) {

            // ajouter le paiement au panier
            $panier->initPaiement();
            $paiement = new Paiement($this->request->get('paiement_id'));

            if ($paiement instanceof Paiement) {

                if (empty($paiement->paiement_type)) {
                    throw new DediException('no paiement type');
                }

                $panier->addPayment($aPayerTTC, $paiement);
                $panier->setDeviceType($this->getDeviceType());

                if ($paiement->paiement_type == Paiement::PASSIF) {
                    try {
                        $commande = new Commande();
                        $session->set('commande_id', $commande->saveFromPanier($panier));
                    } catch (DediException $e) {
                        $flashBag->add('error', $e->getMessage());
                    }

                    if ($session->has('commande_id')) {
                        // On vide tout
                        $panier->init();
                        $response = $this->redirect($this->generateUrl('paiement_success'));
                    }
                }

                if ($paiement->paiement_type == Paiement::ACTIF) {
                    $tpe = new Tpe();
                    $reference = $tpe->generateReferenceFromPanier($this->getSession(), $panier);

                    if (!empty($reference)) {
                        $panier->setReference($reference);
                        $response = $this->redirect($this->generateUrl('paiement_actif'));
                    } else {
                        $flashBag->add(
                            'error',
                            $texte->getOneOrCreate('errGenerateReference', 'Une erreur technique est survenue')
                        );
                    }
                }

            } else {
                $flashBag->add(
                    'error',
                    $texte->getOneOrCreate(
                        'errPaiementSelected',
                        'Une erreur est survenue avec le mode de paiement selectionné, veuillez en choisir un autre'
                    )
                );
            }

        }

        if ($flashBag->has('error')) {
            $response = new RedirectResponse($this->generateUrl('order'));
        }

        return $response;
    }

    /**
     * @Route(
     *      path         = "/pdf/livraison/",
     *      name         = "bon_livraison"
     * )
     */
    public function pdf_bl()
    {
        $arrCommande = $this->getInfosPdf();

        $htmlContent = $this->template->fetch('pdf/bl.tpl');
        $fileName = 'BL' . $arrCommande[0]['commande_id'];

        return $this->renderPdf($htmlContent, $fileName);

    }

    /**
     * @Route(
     *      path         = "/pdf/commande/",
     *      name         = "bon_commande"
     * )
     */
    public function pdf_bc()
    {
        $arrCommande = $this->getInfosPdf();

        $htmlContent = $this->template->fetch('pdf/bc.tpl');
        $fileName = 'BC' . $arrCommande[0]['commande_id'];

        return $this->renderPdf($htmlContent, $fileName);
    }

    /**
     * @Route(
     *      path         = "/pdf/facture/",
     *      name         = "facture"
     * )
     */
    public function pdf_facture()
    {

        $arrCommande = $this->getInfosPdf();

        if (empty($arrCommande[0]['commande_facture_numero'])) {
            throw new ResourceNotFoundException();
        }

        $htmlContent = $this->template->fetch('pdf/fc.tpl');
        $fileName = 'FC' . $arrCommande[0]['commande_facture_numero'];

        return $this->renderPdf($htmlContent, $fileName);
    }

    /**
     * @Route(
     *      path         = "/pdf/avoir/",
     *      name         = "avoir"
     * )
     */
    public function pdf_avoir()
    {

        $avoir = new Avoir;
        $client = new Client;
        $adresse = new Adresse;

        $arrAvoir = $avoir->get(array('avoir_id' => $this->request->get('avoir_id'),'commande_id' => $this->request->get('commande_id')));
        if(empty($arrAvoir)) {
            throw new ResourceNotFoundException();
        }

        $arrClient = $client->get(array('client_id' => $arrAvoir[0]['client_id']));

        $arrAdresse = $adresse->get(array('client_id' => $arrAvoir[0]['client_id'], 'adresse_default' => 1));

        $this->template->assign('avoir',$arrAvoir[0]);
        $this->template->assign('client',$arrClient[0]);
        $this->template->assign('adresse',$arrAdresse[0]);


        $htmlContent = $this->template->fetch('pdf/avoir.tpl');
        $fileName = 'AV' . $arrAvoir[0]['avoir_id'];

        return $this->renderPdf($htmlContent, $fileName);
    }

    protected function getInfosPdf()
    {
        $commande = new Commande;
        $client = new Client;

        // on recupere la commande si pas de commande on sort
        $arrCommande = $commande->get(array('commande_id' => $this->request->get('commande_id'), 'client_id' => $this->request->get('client_id')));
        if (count($arrCommande) == 0) {
            throw new ResourceNotFoundException();
        }

        // on recupere les produits
        $arrCmdProduit = $commande->getProduit(array('commande_id' => $this->request->get('commande_id')));

        //on récupère la totalité des infos client
        $arrClient = $client->get(array('client_id' => $this->request->get('client_id')));

        $arrCommande[0]['client_date_inscription'] = $arrClient[0]['date_inscription'];

        //Tableau des Taxes
        $Taxes = array();
        $key=0;

        foreach($arrCmdProduit as $p) {
            //Recuperation des differentes TVA presentes dans la commande
            $ins = true;
            foreach($Taxes as $t) {
                if($t['taux_taxe'] == $p['produit_taxe']) {
                    $ins = false;
                }
            }
            if($ins && $p['produit_taxe'] != 0) {
                $Taxes[$key]['taux_taxe'] = $p['produit_taxe'];
                $Taxes[$key]['base_ht'] = 0;
                $Taxes[$key]['montant_tva'] = 0;
                $key++;
            }
        }
        $nbTaxe = count($Taxes);

        if($nbTaxe == 0) {
            //Si ya pas de taxe, on affiche rien
            $this->template->assign('no_taxe',1);
        }
        else {
            //Verifie si les fdp sont soumise a des taxes, si oui et que c'est different a une des taxes, on l'ajoute au tableau
            $ins2 = true;
            foreach($Taxes as $t) {
                if($arrCommande[0]['commande_port_taxe'] == $t['taux_taxe'] || $arrCommande[0]['commande_port_taxe'] == 0) {
                    $ins2 = false;
                }
            }
            if($ins2) {
                $Taxes[$nbTaxe]['taux_taxe'] = $arrCommande[0]['commande_port_taxe'];
                $Taxes[$nbTaxe]['base_ht'] = 0;
                $Taxes[$nbTaxe]['montant_tva'] = 0;
            }

            //CALCUL DU TABLEAU DE TVA
            foreach($Taxes as $key => $value) {
                foreach($arrCmdProduit as $p) {
                    if($Taxes[$key]['taux_taxe'] == $p['produit_taxe']) {
                        $Taxes[$key]['base_ht'] = $Taxes[$key]['base_ht'] + $p['produit_total_ht'];
                    }
                }
                if($arrCommande[0]['commande_port_taxe'] == $Taxes[$key]['taux_taxe']) {
                    $Taxes[$key]['base_ht'] = $Taxes[$key]['base_ht'] + $arrCommande[0]['commande_port_ht'];
                }
                $Taxes[$key]['montant_tva'] = $Taxes[$key]['base_ht']*$Taxes[$key]['taux_taxe'];
            }

            $this->template->assign('taxe',$Taxes);
        }
        //-- Tableau des Taxes

        // on assigne
        $this->template->assign('commande', $arrCommande[0]);
        $this->template->assign('produit', $arrCmdProduit);
        $this->template->assign('SITE_URL', SITE_URL);
        $this->template->assign('SITE_NAME', SITE_NAME);

        // Gestion des commandes HT
        if ($arrCommande[0]['commande_taxe'] == 2) {
            $this->template->assign('isTva', false);
        } else {
            $this->template->assign('isTva', true);
        }

        return $arrCommande;
    }

    /**
     * @Route(
     *      path         = "/pdf/proforma/",
     *      name         = "proforma"
     * )
     */
    public function pdf_proforma()
    {
        $arrCommande = $this->getInfosPdf();

        $htmlContent = $this->template->fetch('pdf/proforma.tpl');
        $fileName = 'Proforma' . $arrCommande[0]['commande_id'];

        return $this->renderPdf($htmlContent, $fileName);
    }

    /**
     * @Route(
     *      path         = "/pdf/multifac/",
     *      name         = "multifac"
     * )
     */
    public function pdf_multifac()
    {
        if ($this->request->get('name')) {
            header("Content-type: application/force-download");
            header("Content-Disposition: attachment; filename=" . $this->request->get('name') . ".pdf");
            readfile(SITE_DIR . 'export/' . $this->request->get('name') . '.pdf');
        }

        //return $this->renderPdf($htmlContent, $fileName);
    }

    /**
     * @Route(
     *      path         = "/order_exist.html",
     *      name         = "order_exist",
     *      methods      = { "POST" },
     *      defaults     = {"role" = "customer"}
     * )
     *
     * @return mixed
     */
    public function checkOrder()
    {
        $cart = Panier::getInstance();
        $tpe = new Tpe($cart->getReference());

        $json['hasOrder'] = !empty($tpe->commande_id);

        return $this->response($json);
    }
}