<?php

namespace Controller;

use Breadcrumb;
use Client;
use Panier;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Adresse;
use Livraison;
use Paiement;
use Texte;
use Tpe;
use Commande;
use DediException;
use Avoir;
use BonAchat;
use Pays;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class RecapitulatifController
 * @package Controller
 */
class RecapitulatifController extends Controller {

    /**
     * @Route(
     *      path         = "/checkout_recapitulatif.html",
     *      name         = "recapitulatif",
     *      methods      = { "GET" },
     *      defaults     = {"role" = "customer"}
     * )
     *
     * @return mixed
     */
    public function show()
    {

        $panier = Panier::getInstance();
        $tpl_vars = [];
        $client = $this->getCurrentClient();
        $bonAchat = new BonAchat();
        $session = $this->getSession();

        list($tpl_vars['sTotalHT'], $tpl_vars['sTotalTTC']) = $panier->getTotalProducts();
        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();

        $caddie = $panier->getProducts();
        $tpl_vars['caddie'] = $caddie;

        // recup mode de paiement
        $paiement = new Paiement();
        $paramPay = [];
        if ($session->has('token') && $session->has('paymentId') && $session->has('PayerID')) {
            $paramPay['paiement_id'] = Paiement::PAYPAL_EXPRESS;
            $paramPay['all'] = true;
            $tpl_vars['paypal_express'] = true;
        } else {
            $paramPay['prix'] = $aPayerHT;
        }
        $tpl_vars['paiements'] = $paiement->get($paramPay);

        $reduction = $panier->getReduction();
        if (!empty($reduction)) {
            $this->template->assign('codePromo', $reduction->code);
            $this->template->assign('montantReduc', $reduction->price);
        }

        // on recup les ba valide et qui peuvent etre utilisé
        $this->template->assign('bonAchats', $bonAchat->get(array(
            'client_id' => $client->client_id,
            'bon_achat_statut_id' => BonAchat::BA_VALIDE,
            'interval' => array(date('Y-m-d'))
        )));

        // on recup les BA deja selectionner
        $baSelected = $panier->getBonAchat();
        $listBaSelected = array();
        $totalBa = 0;
        if (!empty($baSelected)) {
            foreach ($baSelected as $baS) {
                $listBaSelected[] = $baS->bon_achat_id;
                $totalBa+= $baS->bon_achat_prix;
            }
        }

        $shipping = $panier->getShipping();
        $this->template->assign('port', $shipping->toArray());

        $this->template->assign('listBaSelected', $baSelected);
        $this->template->assign('bonAchatsSelected', $listBaSelected);
        $this->template->assign('totalBa', $totalBa);

        $tpl_vars['aPayerHT'] = $aPayerHT;
        $tpl_vars['aPayerTTC'] = $aPayerTTC;
        $tpl_vars['montantTva'] = $aPayerTTC - $aPayerHT;

        $flashBag = $this->getFlashBag();
        if ($flashBag->has('error')) {
            $tpl_vars['error'] = $flashBag->get('error');
        }

        return $this->render('recapitulatif.tpl', $tpl_vars);
    }

    /**
     * @Route(
     *      path         = "/checkout_recapitulatif.html",
     *      name         = "recapitulatif_post",
     *      methods      = { "POST" },
     *      defaults     = { "role" = "customer" }
     * )
     *
     * @return mixed
     */
    public function post()
    {

        $flashBag = $this->getFlashBag();
        $texte = new Texte();
        $panier = Panier::getInstance();
        $session = $this->getSession();

        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();

        if ($this->request->get('paiement_id') == false && $aPayerHT > 0) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errSelectPaiement', 'Vous devez selectionner un mode de paiement')
            );
        }

        if ($this->request->get('cgv') == false) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errValidCGV', 'Vous devez valider les CGV pour passer au paiement')
            );
        }

        if ($flashBag->has('error') == false) {

            // ajouter le paiement au panier
            $panier->initPaiement();
            $paiement = new Paiement($this->request->get('paiement_id'));

            if ($paiement instanceof Paiement) {

                $panier->addPayment($aPayerTTC, $paiement);
                $panier->setDeviceType($this->getDeviceType());

                if ($paiement->paiement_type == Paiement::PASSIF) {
                    try {
                        $commande = new Commande();
                        $session->set('commande_id', $commande->saveFromPanier($panier));
                    } catch (DediException $e) {
                        $flashBag->add('error', $e->getMessage());
                    }

                    if ($session->has('commande_id')) {
                        // On vide tout
                        $panier->init();
                        $response = $this->redirect($this->generateUrl('paiement_success'));
                    }
                }

                if ($paiement->paiement_type == Paiement::ACTIF) {

                    $tpe = new Tpe();
                    $reference = $tpe->generateReferenceFromPanier($this->getSession(), $panier);

                    if (!empty($reference)) {
                        $panier->setReference($reference);
                        $response = $this->redirect($this->generateUrl('paiement_actif'));
                    } else {
                        $flashBag->add(
                            'error',
                            $texte->getOneOrCreate('errGenerateReference', 'Une erreur technique est survenue')
                        );
                    }
                }

            } else {
                $flashBag->add(
                    'error',
                    $texte->getOneOrCreate('errPaiementSelected', 'Une erreur est survenue avec le mode de paiement selectionné, veuillez en choisir un autre')
                );
            }
        }

        if ($flashBag->has('error')) {
            $response = $this->redirect(
                $this->generateUrl('recapitulatif')
            );
        }

        return $response;
    }
}
