<?php

namespace DebugBar\DataCollector;

use Panier;

class CartDataCollector extends DataCollector implements Renderable
{
    public function collect()
    {
        $panier = Panier::getInstance();

        $data = array(
            "produits" => $panier->getProducts(),
            "livraison" => array(),
            "facturation" => array(),
            "port" => array(),
            "paiement" => array(),
            "reduction" => array(),
        );

        $addrShip = $panier->getAddressShipping();
        if ($addrShip instanceof \Adresse) {
            $data['livraison'] = $addrShip->toArray();
        }

        $addrBill = $panier->getAddressBilling();
        if ($addrShip instanceof \Adresse) {
            $data['facturation'] = $addrBill->toArray();
        }

        $ship = $panier->getShipping();
        if ($ship instanceof \Livraison) {
            $data['port'] = $ship->toArray();
        }

        $customer = $panier->getCustomer();
        if ($customer instanceof \Client) {
            $data['client'] = $customer->toArray();
        }

        $payments = $panier->getPayment();
        if (!empty($payments)) {
            foreach ($payments as $pay) {
                $data['paiement'][] = $pay->toArray();
            }
        }

        $reduction = $panier->getReduction();
        if (!empty($reduction)) {
            $data['reduction'] = $reduction->toArray();
        }

        foreach ($data as $k => $v) {
            if (!is_string($v)) {
                $v = $this->getDataFormatter()->formatVar($v);
            }
            $data[$k] = $v;
        }

        return $data;
    }

    public function getName()
    {
        return 'Panier';
    }

    public function getWidgets()
    {
        $name = $this->getName();
        return array(
            "$name" => array(
                "icon" => "gear",
                "widget" => "PhpDebugBar.Widgets.VariableListWidget",
                "map" => "$name",
                "default" => "{}"
            )
        );
    }
}