<?php

namespace Form;

use Form\Constraint\Callback;
use Form\Constraint\Date;
use Form\Constraint\EqualToField;
use Form\Constraint\Length;
use Form\Constraint\Mail;
use Form\Constraint\NotBlank;

class RegisterForm extends Form
{
    function __construct()
    {
        parent::__construct();
        $texte = new \Texte();

        $this
            // email
            ->add(
                new Field(
                    "client_email",
                    array(
                        "not_empty"  => new NotBlank(),
                        "is_mail"    => new Mail(),
                        "dispo_mail" => new Callback(array("Client", "mailIsFree")),
                    ),
                    array(
                        "not_empty"  => $texte->getOneOrCreate("email_obligatoire", "Vous devez remplir le champs email."),
                        "is_mail"    => $texte->getOneOrCreate("email_invalide", "Vous devez utiliser un email correct."),
                        "dispo_mail" => $texte->getOneOrCreate("email_deja_utilise", "L'adresse email que vous avez renseigné est déjà utilisée."),
                    )
                )
            )
            // password
            ->add(
                new Field(
                    "client_password",
                    array(
                        "not_empty"  => new NotBlank(),
                        "min_length" => new Length(4, 255),
                    ),
                    array(
                        "not_empty" => $texte->getOneOrCreate("email_confirmation_obligatoire", "Vous devez remplir le champ de confirmation d'email."),
                        "equal"     => $texte->getOneOrCreate("email_confirmation_identique", "Votre email et sa confirmation doivent être identiques."),
                    )
                )
            )
            // password confirm
            ->add(
                new Field(
                    "client_password_confirm",
                    array(
                        "not_empty" => new NotBlank(),
                        "equal"     => new EqualToField("client_password"),
                    ),
                    array(
                        "not_empty" => $texte->getOneOrCreate("password_confirmation_obligatoire", "Vous devez remplir le champ de confirmation de mot de passe."),
                        "equal"     => $texte->getOneOrCreate("password_confirmation_identique", "Le mot de passe et sa confirmation doivent être identiques."),
                    )
                )
            )
            // civilite
            ->add(
                new Field(
                    "civilite_id",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => $texte->getOneOrCreate("civilite_obligatoire", "Vous devez choisir une civilité."))
                )
            )
            // nom
            ->add(
                new Field(
                    "client_nom",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => $texte->getOneOrCreate("nom_obligatoire", "Vous devez remplir le champ nom"))
                )
            )
            // prenom
            ->add(
                new Field(
                    "client_prenom",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => $texte->getOneOrCreate("prenom_obligatoire", "Vous devez remplir le champ prénom"))
                )
            )
            // date de naissance
            ->add(
                new Field(
                    "client_ddn",
                    array(
                        "not_empty" => new NotBlank(),
                        "date_ddn"  => new Date("Y-m-d"),
                    ),
                    array(
                        "not_empty" => $texte->getOneOrCreate("ddn_obligatoire", "Vous devez remplir une date de naissance."),
                        "date_ddn"  => $texte->getOneOrCreate("ddn_invalide", "Date de naissance invalide."),
                    )
                )
            )
            //rue
            ->add(
                new Field(
                    "adresse_rue",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => $texte->getOneOrCreate("rue_obligatoire", "Vous devez remplir le champ rue."))
                )
            )
            // code postal
            ->add(
                new Field(
                    "adresse_cp",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => $texte->getOneOrCreate("cp_obligatoire", "Vous devez remplir le champ code postal."))
                )
            )
            // ville
            ->add(
                new Field(
                    "adresse_ville",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => $texte->getOneOrCreate("ville_obligatoire", "Vous devez remplir le champ ville."))
                )
            )
            // pays
            ->add(
                new Field(
                    "pays_id",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => $texte->getOneOrCreate("pays_obligatoire", "Vous devez choisir un pays."))
                )
            )
            // telephone
            ->add(
                new Field(
                    "client_telephone",
                    array("not_empty" => new NotBlank(),),
                    array("not_empty" => $texte->getOneOrCreate("telephone_obligatoire", "Vous devez remplir le numéro de téléphone."),)
                )
            )
        ;
    }

}